@@ -791,6 +791,10 @@ viterbi.msm <- function(x)
                 as.integer (x$model$covmatch),
                 as.integer (x$model$ndeath),                
                 as.integer (x$model$death),
+                as.integer(x$model$cens$ncens),
+                as.integer(x$model$cens$censor - 1),
+                as.integer(x$model$cens$states - 1),
+                as.integer(x$model$cens$index - 1),
                 as.integer (x$model$exacttimes),
                 as.integer(0),
                 as.integer(-1),
